VERSION 5.00
Begin VB.UserControl SystemButton 
   ClientHeight    =   240
   ClientLeft      =   0
   ClientTop       =   0
   ClientWidth     =   240
   ScaleHeight     =   16
   ScaleMode       =   3  'Pixel
   ScaleWidth      =   16
   ToolboxBitmap   =   "SystemButton.ctx":0000
End
Attribute VB_Name = "SystemButton"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
Option Explicit

Private Type RECT
    Left As Long
    Top As Long
    Right As Long
    Bottom As Long
End Type

Private Declare Function DrawFrameControl Lib "user32" (ByVal hDC As Long, lpRect As RECT, ByVal un1 As Long, ByVal un2 As Long) As Long

' Frame type.
Public Enum sb_ButtonStyle
    DFC_BUTTON = 4
    DFC_CAPTION = 1
    DFC_MENU = 2
    DFC_POPUPMENU = 5
    DFC_SCROLL = 3
End Enum

' Frame states.
Public Enum sb_ButtonType
    ' DFC_BUTTON
    DFCS_BUTTON3STATE = &H8
    DFCS_BUTTONCHECK = &H0
    DFCS_BUTTONPUSH = &H10
    DFCS_BUTTONRADIO = &H4
    DFCS_BUTTONRADIOIMAGE = &H1
    DFCS_BUTTONRADIOMASK = &H2

    ' DFC_CAPTION
    DFCS_CAPTIONCLOSE = &H0
    DFCS_CAPTIONHELP = &H4
    DFCS_CAPTIONMAX = &H2
    DFCS_CAPTIONMIN = &H1
    DFCS_CAPTIONRESTORE = &H3

    ' DFC_MENU
    DFCS_MENUARROW = &H0
    DFCS_MENUARROWRIGHT = &H4
    DFCS_MENUBULLET = &H2
    DFCS_MENUCHECK = &H1

    ' DFC_SCROLL
    DFCS_SCROLLCOMBOBOX = &H5
    DFCS_SCROLLDOWN = &H1
    DFCS_SCROLLLEFT = &H2
    DFCS_SCROLLRIGHT = &H3
    DFCS_SCROLLSIZEGRIP = &H8
    DFCS_SCROLLSIZEGRIPRIGHT = &H10
    DFCS_SCROLLUP = &H0

    DFCS_ADJUSTRECT = &H2000
End Enum

' Statuses.
Private Const DFCS_CHECKED = &H400
Private Const DFCS_FLAT = &H4000
Private Const DFCS_HOT = &H1000
Private Const DFCS_INACTIVE = &H100
Private Const DFCS_MONO = &H8000
Private Const DFCS_NOT_PUSHED = &H0
Private Const DFCS_PUSHED = &H200

' Default property values.
Private Const m_def_ButtonStyle = DFC_CAPTION
Private Const m_def_ButtonType = DFCS_CAPTIONCLOSE
Private Const m_def_Hot = False
Private Const m_def_Flat = False
Private Const m_def_Mono = False
Private Const m_def_Checked = False

' Property values.
Private m_ButtonStyle As sb_ButtonStyle
Private m_ButtonType As sb_ButtonType
Private m_Hot As Boolean
Private m_Flat As Boolean
Private m_Mono As Boolean
Private m_Checked As Boolean

Private m_Pressing As Boolean
Private m_Pressed As Boolean

Public Event Click()
Private Sub DrawControl()
Dim frame_status As Long

    If Not Enabled Then frame_status = frame_status Or DFCS_INACTIVE
    If m_Pressed Then frame_status = frame_status Or DFCS_PUSHED

    If m_Hot Then frame_status = frame_status Or DFCS_HOT
    If m_Flat Then frame_status = frame_status Or DFCS_FLAT
    If m_Mono Then frame_status = frame_status Or DFCS_MONO
    If m_Checked Then frame_status = frame_status Or DFCS_CHECKED

    DrawFrame UserControl.hDC, 0, 0, _
        ScaleWidth, ScaleHeight, _
        m_ButtonStyle, m_ButtonType, _
        frame_status
End Sub
' Draw a frame control image.
Private Sub DrawFrame(ByVal hDC As Long, ByVal X As Long, ByVal Y As Long, ByVal wid As Long, ByVal hgt As Long, ByVal frame_type As Long, ByVal frame_state As Long, Optional ByVal frame_status As Long = 0)
Dim r As RECT

    With r
        .Left = X
        .Right = X + wid
        .Top = Y
        .Bottom = Y + hgt
    End With

    DrawFrameControl hDC, r, frame_type, frame_state Or frame_status
End Sub

Public Property Get ButtonStyle() As sb_ButtonStyle
    ButtonStyle = m_ButtonStyle
End Property

Public Property Let ButtonStyle(ByVal New_ButtonStyle As sb_ButtonStyle)
    m_ButtonStyle = New_ButtonStyle
    PropertyChanged "ButtonStyle"

    DrawControl
End Property

Public Property Get ButtonType() As sb_ButtonType
    ButtonType = m_ButtonType
End Property

Public Property Let ButtonType(ByVal New_ButtonType As sb_ButtonType)
    m_ButtonType = New_ButtonType
    PropertyChanged "ButtonType"

    DrawControl
End Property

' Initialize properties.
Private Sub UserControl_InitProperties()
    m_ButtonStyle = m_def_ButtonStyle
    m_ButtonType = m_def_ButtonType
    m_Hot = m_def_Hot
    m_Flat = m_def_Flat
    m_Checked = m_def_Checked
    m_Mono = m_def_Mono
End Sub

' Start a button press.
Private Sub UserControl_MouseDown(Button As Integer, Shift As Integer, X As Single, Y As Single)
    m_Pressing = True
    m_Pressed = True
    DrawControl
End Sub


' Continue a button press.
Private Sub UserControl_MouseMove(Button As Integer, Shift As Integer, X As Single, Y As Single)
Dim new_pressed As Boolean

    If Not m_Pressing Then Exit Sub

    ' See if the mouse is over the control.
    new_pressed = (X >= 0 And X <= ScaleWidth And _
        Y >= 0 And Y <= ScaleHeight)

    ' Redraw the control if necessary.
    If m_Pressed <> new_pressed Then
        m_Pressed = new_pressed
        DrawControl
    End If
End Sub


' Stop a button press.
Private Sub UserControl_MouseUp(Button As Integer, Shift As Integer, X As Single, Y As Single)
    ' Raise the Click event if the button
    ' is currently pressed.
    If m_Pressed Then RaiseEvent Click

    ' Stop the press.
    m_Pressing = False
    m_Pressed = False

    DrawControl
End Sub


Private Sub UserControl_Paint()
    DrawControl
End Sub


' Load saved property values.
Private Sub UserControl_ReadProperties(PropBag As PropertyBag)
    m_ButtonStyle = PropBag.ReadProperty("ButtonStyle", m_def_ButtonStyle)
    m_ButtonType = PropBag.ReadProperty("ButtonType", m_def_ButtonType)
    UserControl.Enabled = PropBag.ReadProperty("Enabled", True)
    m_Hot = PropBag.ReadProperty("Hot", m_def_Hot)
    m_Flat = PropBag.ReadProperty("Flat", m_def_Flat)
    m_Checked = PropBag.ReadProperty("Checked", m_def_Checked)
    m_Mono = PropBag.ReadProperty("Mono", m_def_Mono)
End Sub

' Save property values.
Private Sub UserControl_WriteProperties(PropBag As PropertyBag)
    PropBag.WriteProperty "ButtonStyle", m_ButtonStyle, m_def_ButtonStyle
    PropBag.WriteProperty "ButtonType", m_ButtonType, m_def_ButtonType
    PropBag.WriteProperty "Enabled", UserControl.Enabled, True
    PropBag.WriteProperty "Hot", m_Hot, m_def_Hot
    PropBag.WriteProperty "Flat", m_Flat, m_def_Flat
    PropBag.WriteProperty "Checked", m_Checked, m_def_Checked
    PropBag.WriteProperty "Mono", m_Mono, m_def_Mono
End Sub

Public Property Get Checked() As Boolean
    Checked = m_Checked
End Property
Public Property Let Checked(ByVal New_Checked As Boolean)
    m_Checked = New_Checked
    PropertyChanged "Checked"

    DrawControl
End Property
Public Property Get Mono() As Boolean
    Mono = m_Mono
End Property
Public Property Let Mono(ByVal New_Mono As Boolean)
    m_Mono = New_Mono
    PropertyChanged "Mono"

    DrawControl
End Property
Public Property Get Flat() As Boolean
    Flat = m_Flat
End Property
Public Property Let Flat(ByVal New_Flat As Boolean)
    m_Flat = New_Flat
    PropertyChanged "Flat"

    DrawControl
End Property
Public Property Get Enabled() As Boolean
Attribute Enabled.VB_Description = "Returns/sets a value that determines whether an object can respond to user-generated events."
    Enabled = UserControl.Enabled
End Property

Public Property Let Enabled(ByVal New_Enabled As Boolean)
    UserControl.Enabled() = New_Enabled
    PropertyChanged "Enabled"

    DrawControl
End Property

Public Property Get Hot() As Boolean
    Hot = m_Hot
End Property

Public Property Let Hot(ByVal New_Hot As Boolean)
    m_Hot = New_Hot
    PropertyChanged "Hot"

    DrawControl
End Property
